@@ -13,33 +13,37 @@
 #include "toplevel.h"
 
 #include <kmessagebox.h>
-#include <kfiledialog.h>
-#include <klocale.h>
 #include <KLocalizedString>
-#include <kio/netaccess.h>
 #include <kstandardaction.h>
 #include <kstandardshortcut.h>
 #include <kstandardgameaction.h>
 #include <kactioncollection.h>
 #include <ktoggleaction.h>
 #include <ktogglefullscreenaction.h>
-#include <kimageio.h>
-#include <kmimetype.h>
 #include <kconfiggroup.h>
 #include <kcombobox.h>
+#include <ksharedconfig.h>
+#include <kio/job.h>
 
 #include <QApplication>
 #include <QClipboard>
+#include <QFileDialog>
 #include <QFileInfo>
+#include <QImageWriter>
+#include <QMimeDatabase>
 #include <QPrintDialog>
 #include <QPrinter>
 #include <QTemporaryFile>
 #include <QWidgetAction>
 
+#include "filefactory.h"
 #include "playground.h"
 #include "soundfactory.h"
 #include "playgrounddelegate.h"
 
+// TODO kdelibs4support REMOVE
+#include <KLocale>
+
 static const char *DEFAULT_THEME = "default_theme.theme";
 
 // Constructor
@@ -48,7 +52,7 @@ TopLevel::TopLevel()
 {
   QString board, language;
 
-  playGround = new PlayGround(this);
+  playGround = new PlayGround(this, this);
   playGround->setObjectName( QStringLiteral( "playGround" ) );
 
   soundFactory = new SoundFactory(this);
@@ -142,7 +146,7 @@ void TopLevel::changeGameboard(const QString &newGameBoard)
   QFileInfo fi(newGameBoard);
   if (fi.isRelative())
   {
-    fileToLoad = QStandardPaths::locate(QStandardPaths::AppDataLocation, QLatin1String( "pics/" ) + newGameBoard);
+    fileToLoad = FileFactory::locate(QLatin1String( "pics/" ) + newGameBoard);
   }
   else
   {
@@ -189,7 +193,7 @@ void TopLevel::changeLanguage(const QString &soundFile)
   QFileInfo fi(soundFile);
   if (fi.isRelative())
   {
-    fileToLoad = QStandardPaths::locate(QStandardPaths::AppDataLocation, QLatin1String( "sounds/" ) + soundFile);
+    fileToLoad = FileFactory::locate(QLatin1String( "sounds/" ) + soundFile);
   }
   else
   {
@@ -213,7 +217,7 @@ void TopLevel::changeLanguage(const QString &soundFile)
 }
 
 // Play a sound
-void TopLevel::playSound(const QString &ref) const
+void TopLevel::playSound(const QString &ref)
 {
   soundFactory->playSound(ref);
 }
@@ -331,8 +335,8 @@ void TopLevel::fileNew()
 // Load gameboard
 void TopLevel::fileOpen()
 {
-  QUrl url = KFileDialog::getOpenUrl(QUrl(QStringLiteral("kfiledialog:///<ktuberling>")),
-                                     QStringLiteral("*.tuberling|%1\n*|%2").arg(i18n("KTuberling files"), i18n("All files")));
+  QUrl url = QFileDialog::getOpenFileUrl(this, i18n("Load file"), QUrl(),
+                                     i18n("KTuberling files (%1)", QStringLiteral("*.tuberling")));
 
   open(url);
 }
@@ -343,8 +347,20 @@ void TopLevel::open(const QUrl &url)
     return;
 
   QString name;
-
-  KIO::NetAccess::download(url, name, this);
+  if (url.isLocalFile()) {
+    // file protocol. We do not need the network
+    name = url.toLocalFile();
+  } else {
+    QTemporaryFile tmpFile;
+    tmpFile.setAutoRemove(false);
+    tmpFile.open();
+    name = tmpFile.fileName();
+    const QUrl dest = QUrl::fromLocalFile(name);
+    KIO::Job *job = KIO::file_copy(url, dest, -1, KIO::Overwrite | KIO::HideProgressInfo);
+    QEventLoop eventLoop;
+    connect(job, &KIO::Job::result, &eventLoop, &QEventLoop::quit);
+    eventLoop.exec(QEventLoop::ExcludeUserInputEvents);
+  }
 
   switch(playGround->loadFrom(name))
   {
@@ -361,16 +377,15 @@ void TopLevel::open(const QUrl &url)
     break;
   }
 
-
-  KIO::NetAccess::removeTempFile( name );
+  if (!url.isLocalFile()) {
+    QFile::remove(name);
+  }
 }
 
 // Save gameboard
 void TopLevel::fileSave()
 {
-  KUrl url = KFileDialog::getSaveUrl
-                ( KUrl("kfiledialog:///<ktuberling>"),
-                  QStringLiteral("*.tuberling|%1").arg(i18n("KTuberling files")), this, QString(), KFileDialog::ConfirmOverwrite);
+  QUrl url = QFileDialog::getSaveFileUrl( this, QString(), QUrl(), i18n("KTuberling files (%1)", QStringLiteral("*.tuberling")) );
 
   if (url.isEmpty())
     return;
@@ -399,7 +414,7 @@ void TopLevel::fileSave()
 
   if( !url.isLocalFile() )
   {
-    if (!KIO::NetAccess::upload(name, url, this))
+    if (!upload(name, url))
       KMessageBox::error(this, i18n("Could not save file."));
   }
 }
@@ -407,19 +422,33 @@ void TopLevel::fileSave()
 // Save gameboard as picture
 void TopLevel::filePicture()
 {
-  const QString patternsString = KImageIO::pattern(KImageIO::Writing);
-  QStringList patterns = patternsString.split(QStringLiteral("\n"));
-  // Favor png
-  if (!patterns.isEmpty()) {
-      QString firstLine = patterns[0];
-      patterns.removeAt(0);
-      if (firstLine.contains(QStringLiteral(" *.png"))) {
-          firstLine.remove(QStringLiteral(" *.png"));
-          firstLine.prepend("*.png ");
+  const QMimeDatabase mimedb;
+  const QList<QByteArray> imageWriterMimetypes = QImageWriter::supportedMimeTypes();
+  QStringList patterns;
+  for(auto mimeName : imageWriterMimetypes)
+  {
+    const QMimeType mime = mimedb.mimeTypeForName(mimeName);
+    if (mime.isValid())
+    {
+      QStringList suffixes;
+      for(const QString &suffix : mime.suffixes())
+      {
+          suffixes << QString("*.%1").arg(suffix);
       }
-      patterns.prepend(firstLine);
+
+      // Favor png
+      const QString pattern = i18nc("%1 is mimetype and %2 is the file extensions", "%1 (%2)", mime.comment(), suffixes.join(' '));
+      if (mimeName == "image/png")
+      {
+        patterns.prepend(pattern);
+      }
+      else
+      {
+        patterns << pattern;
+      }
+    }
   }
-  const KUrl url = KFileDialog::getSaveUrl(KUrl(QStringLiteral("kfiledialog:///<ktuberling>")), patterns.join(QStringLiteral("\n")), this, QString(), KFileDialog::ConfirmOverwrite);
+  const QUrl url = QFileDialog::getSaveFileUrl( this, QString(), QUrl(), patterns.join(";;") );
 
   if( url.isEmpty() )
     return;
@@ -436,19 +465,9 @@ void TopLevel::filePicture()
     name = url.path();
   }
 
-  KMimeType::Ptr mime = KMimeType::findByUrl(url, 0, true, true);
-  if (!KImageIO::isSupported(mime->name(), KImageIO::Writing))
-  {
-    KMessageBox::error(this, i18n("Unknown picture format."));
-    return;
-  };
-
-  QStringList types = KImageIO::typeForMime(mime->name());
-  if (types.isEmpty()) return; // TODO error dialog?
-
   QPixmap picture(playGround->getPicture());
 
-  if (!picture.save(name, types.at(0).toLatin1()))
+  if (!picture.save(name))
   {
     KMessageBox::error
       (this, i18n("Could not save file."));
@@ -457,7 +476,7 @@ void TopLevel::filePicture()
 
   if( !url.isLocalFile() )
   {
-    if (! KIO::NetAccess::upload(name, url, this))
+    if (!upload(name, url))
       KMessageBox::error(this, i18n("Could not save file."));
   }
 
@@ -515,3 +534,14 @@ void TopLevel::lockAspectRatio(bool lock)
   playGround->lockAspectRatio(lock);
   writeOptions();
 }
+
+bool TopLevel::upload(const QString &src, const QUrl &target)
+{
+  bool success = true;
+  const QUrl srcUrl = QUrl::fromLocalFile(src);
+  KIO::Job *job = KIO::file_copy(srcUrl, target, -1, KIO::Overwrite | KIO::HideProgressInfo);
+  QEventLoop eventLoop;
+  connect(job, &KIO::Job::result, this, [job, &eventLoop, &success] { success = !job->error(); eventLoop.quit(); } );
+  eventLoop.exec(QEventLoop::ExcludeUserInputEvents);
+  return success;
+}
