@@ -12,14 +12,15 @@
 #define _TOPLEVEL_H_
 
 #include <kxmlguiwindow.h>
-#include <kurl.h>
 #include <kcombobox.h>
 
+#include "soundfactory.h"
+#include "playground.h"
+
 class QActionGroup;
 class PlayGround;
-class SoundFactory;
 
-class TopLevel : public KXmlGuiWindow
+class TopLevel : public KXmlGuiWindow, public SoundFactoryCallbacks, public PlayGroundCallbacks
 {
   Q_OBJECT
 
@@ -29,12 +30,12 @@ public:
   ~TopLevel();
 
   void open(const QUrl &url);
-  void registerGameboard(const QString& menuText, const QString& boardFile, const QPixmap& pixmap);
-  void registerLanguage(const QString &code, const QString &soundFile, bool enabled);
+  void registerGameboard(const QString& menuText, const QString& boardFile, const QPixmap& pixmap) override;
+  void registerLanguage(const QString &code, const QString &soundFile, bool enabled) override;
   void changeLanguage(const QString &langCode);
-  void playSound(const QString &ref) const;
+  void playSound(const QString &ref) override;
 
-  bool isSoundEnabled() const;
+  bool isSoundEnabled() const override;
 
   void changeGameboard(const QString &gameboard);
 
@@ -44,7 +45,7 @@ protected:
   void setupKAction();
 
 protected slots:
-  void saveNewToolbarConfig();
+  void saveNewToolbarConfig() override;
 
 private slots:
 
@@ -62,6 +63,8 @@ private slots:
   void lockAspectRatio(bool lock);
 
 private:
+  bool upload(const QString &src, const QUrl &target);
+
   int                           // Menu items identificators
       newID, openID, saveID, pictureID, printID, quitID,
       copyID, undoID, redoID,
